#ifndef __GRAPH_CLUSTERING_GRAPH_H__
#define __GRAPH_CLUSTERING_GRAPH_H__

#include "graph.h"

/**
 * Build a graph optimized for spectral clustering.
 *
 * Basically a k-NN graph, but ensuring that that the graph is connected
 * so that there are no "connected component" eigenvalues due simply
 * to artifacts of the graph construction.
 *
 * @arg @c V The number of vertices in the graph.
 * @arg @c k The minimum degree of each node.
 * @arg @c max_nodes The maximum number of VP-tree nodes to visit in approximate
 *                   VP-tree.  Set to -1 to perform exact k-NN search.
 * @arg @c distance A function which provides the distance between nodes i and j
 * @arg @c weight Provides the weighting for a pair of nodes i and j. 
 *                May be set to NULL, in which case this will set all edge
 *                weights to 1.
 */
graph *graph_clustering(int V, int k, int max_nodes, void *user_data,
                        double (*distance)(void *user_data, int i, int j),
                        double (*weight)(void *user_data, int i, int j));

/**
 * A clustering graph which links all pairs of within a given distance epsilon.
 */
graph *graph_distance_clustering(
  int V, double epsilon, void *user_data,
  double (*distance)(void *user_data, int i, int j),
  double (*weight)(void *user_data, int i, int j));

#endif /* #ifndef __GRAPH_CLUSTERING_GRAPH_H__ */
