#include "connected_components.h"


// Component code
static size_t get_component(int *component, int u)
{
  if(component[u] != u) {
    get_component(component, component[u]);
    component[u] = component[component[u]];
  }

  return u;
}

#ifndef MIN
#define MIN(a, b) ((a) < (b) ? (a) : (b))
#endif

static void connect(void *p, const edge_t *e)
{
  int *component, u, v;

  component = (int *)p;
  u = e->u;
  v = e->v;

  get_component(component, u);
  get_component(component, v);

  component[u] = component[v] = MIN(component[u], component[v]);
}

int graph_connected_components(graph *G, int *component)
{
  int i, V;
  int *label_mapping, total_components;
  bool components_local;

  V = graph_vertex_count(G);

  // Initialize components
  components_local = (component == NULL);
  if(components_local) {
    component = (int *)malloc(V * sizeof(int));
  }

  for(i = 0; i < V; i++) {
    component[i] = i;
  }

  // Join connected components
  graph_edges_visit(G, &connect, component);

  ///////////// Simplify labels ////////////

  // Start with null mapping
  label_mapping = (int *)malloc(sizeof(int) * V);
  for(i = 0; i < V; i++) {
    label_mapping[i] = 0;
  }

  // Find and perform the mapping
  total_components = 0;
  for(i = 0; i < V; i++) {
    if(label_mapping[component[i]] == 0) {
      label_mapping[component[i]] = ++total_components;
    }

    component[i] = label_mapping[component[i]];
  }

  // Cleanup
  free(label_mapping);
  if(components_local) {
    free(component);
  }

  return total_components;
}
