#include <assert.h>

#include "io.h"
#include "adjacency_csr.h"

graph *graph_read_file(FILE *fp)
{
  int i, V, E, dir, nread;
  edge_t *edges;
  graph *G;

  // Read graph size
  nread = fscanf(fp, "%d %d %d\n", &V, &E, &dir);
  if(nread != 3) {
    return NULL;
  }

  // Read edges
  edges = (edge_t *)malloc(sizeof(edge_t) * E);
  if(edges == NULL) {
    return NULL;
  }

  for(i = 0; i < E; i++) {
    nread = fscanf(fp, "%d %d %lg\n", &(edges[i].u), &(edges[i].v), &(edges[i].w));
    if(nread != 3) {
      free(edges);
      return NULL;
    }
  }

  // Construct graph
  G = graph_adjacency_csr_create((size_t)V, (size_t)E, dir, 1, edges);
  free(edges);

  return G;
}

graph *graph_read_path(const char *path)
{
  FILE *fp;
  graph *G;

  fp = fopen(path, "r");
  if(fp == NULL) {
    return NULL;
  }

  G = graph_read_file(fp);
  fclose(fp);

  return G;
}

static void write_edge(void *data, const edge_t *e)
{
  FILE *fp;

  fp = (FILE *)data;

  fprintf(fp, "%d %d %lg\n", e->u, e->v, e->w);
}

void graph_write_file(FILE *fp, graph *G)
{
  int V, E, dir;

  // Header
  V = graph_vertex_count(G);
  E = graph_edge_count(G);
  dir = graph_directed(G) ? 1 : 0;
  fprintf(fp, "%d %d %d\n", V, E, dir);

  // Edges
  graph_edges_visit(G, write_edge, (void *)fp);
}

void graph_write_path(const char *path, graph *G)
{
  FILE *fp;

  fp = fopen(path, "w+");
  if(fp == NULL) {
    return;
  }

  graph_write_file(fp, G);

  fclose(fp);
}
