#ifndef __KNN_PARALLEL_H__
#define __KNN_PARALLEL_H__

#include "graph.h"

/**
 * Build a k-nearest-neighbor graph among V nodes.
 *
 * @arg @c max_nodes The maximum number of notes to visit during each k-NN
 *                   search.  Exact k-NN used if set to -1.
 * @arg @c distance A function which provides the distance between nodes i and j
 * @arg @c weight Provides the weighting for a pair of nodes i and j. 
 *                May be set to NULL, in which case this will set all edge
 *                weights to 1.
 */
graph *graph_build_knn_parallel(
  int V, int k, int max_nodes,
  int verbose, int nthreads,
  void *user_data,
  double (*distance)(void *user_data, int i, int j),
  double (*weight)(void *user_data, int i, int j));

#endif /* #ifndef __KNN_PARALLEL_H__ */
