#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <sysexits.h>

#include "io.h"
#include "laplacian.h"
#include "util/matio.h"

static const char * const USAGE = "laplacian <graph> <mat>";

int main(int argc, char **argv)
{
  graph *G;
  cs_di *L;
  int s;

  if(argc != 3) {
    fprintf(stderr, "%s\n", USAGE);
    return EX_USAGE;
  }

  // Read in graph
  G = graph_read_path(argv[1]);
  if(G == NULL) {
    fprintf(stderr, "Failed to read graph \"%s\"\n", argv[1]);
    return EX_NOINPUT;
  }

  // Calculate Laplacian
  L = graph_laplacian_sym(G);
  if(L == NULL) {
    fprintf(stderr, "Failed to calculate laplacian.\n");
    return 1;
  }

  // Write out matrix
  s = sparsemat_write_binary_path(argv[2], L);
  if(s != 0) {
    fprintf(stderr, "Failed to write matrix \"%s\"\n", argv[2]);
    return 1;
  }

  return EX_OK;
}
