#include <assert.h>

#include "neighborhood.h"

static int neighbor_compare(const void *p1, const void *p2)
{
  const neighbor_t *n1, *n2;

  n1 = (const neighbor_t *)p1;
  n2 = (const neighbor_t *)p2;

  return n1->v - n2->v;
}

void graph_neighbor_sort(size_t n, neighbor_t *nbrs)
{
  qsort(nbrs, n, sizeof(neighbor_t), &neighbor_compare);
}

neighbor_t *graph_neighbor_bsearch(size_t n, neighbor_t *nbrs, int v)
{
  neighbor_t key;
  void *found;

  key.v = v;

  found = bsearch(&key, nbrs, n, sizeof(neighbor_t), &neighbor_compare);

  return (neighbor_t *)found;
}

neighbor_t *graph_neighbor_lsearch(size_t n, neighbor_t *nbrs, int v)
{
  size_t i;

  for(i = 0; i < n; i++) {
    if(nbrs[i].v == v) {
      return &nbrs[i];
    }
  }

  return NULL;
}

void graph_neighbor_list_set(
  size_t *pd, neighbor_t **pnbrs, bool sort,
  size_t n, const edge_t *edges, bool fwd,
  int exclude)
{
  neighbor_t *nbrs;
  size_t i, j, d;
  bool exists;

  d = *pd;
  nbrs = *pnbrs;

  if(!sort) {
    nbrs = realloc(nbrs, sizeof(neighbor_t) * (d + n));

    for(i = 0; i < n; i++) {
      if((fwd && edges[i].v == exclude) || (!fwd && edges[i].u == exclude)) {
        continue;
      }

      // Handle case the neighbor is already saved
      exists = false;
      for(j = 0; j < d; j++) {
        if((fwd && edges[i].v == nbrs[j].v) || (!fwd && edges[i].u == nbrs[j].v)) {
          exists = true;
          nbrs[j].w = edges[i].w;
          break;
        }
      }

      // Add a new edge
      if(!exists) {
        assert(d < *pd + n);
        if(fwd) {
          nbrs[d].v = edges[i].v;
        }
        else {
          nbrs[d].v = edges[i].u;
        }
        nbrs[d].w = edges[i].w;
        d++;
      }
    }

    // Shrink allocation
    nbrs = realloc(nbrs, sizeof(neighbor_t) * d);
  }
  else {
    // TODO
    assert(0);

  /*
  size_t i, j, k, d;
  neighborhood_t new;

  d = nbr->degree;

  // Allocate new neighborhood
  new.neighbors = (int *)malloc(sizeof(int) * (d + n));
  new.weights = (double *)malloc(sizeof(double) * (d + n));

  printf("begin merge.  n = %lu, d = %lu\n", n, d);

  // Perform a merge operation
  i = j = k = 0;
  while(!(i == n && j == d)) {
    assert(k < n + d);
    printf("i = %lu, j = %lu, k = %lu\n", i, j, k);

    if(j == d || (i < n && e[i].v <= nbr->neighbors[j])) {
      new.neighbors[k] = e[i].v;
      new.weights[k] = e[i].weight;

      i++;
      k++;

      if(j < d && e[i-1].v == nbr->neighbors[j]) {
        j++;
      }
    }
    else {
      new.neighbors[k] = nbr->neighbors[j];
      new.weights[k] = nbr->weights[j];

      j++;
      k++;
    }
  }

  printf("end merge\n\n");

  new.degree = k;
  new.neighbors = (int *)realloc(new.neighbors, sizeof(int) * k);
  new.weights = (double *)realloc(new.weights, sizeof(double) * k);

  *nbr = new;
  */
  }

  *pnbrs = nbrs;
  *pd = d;
}
