#ifndef __GRAPH_NEIGHBORHOOD_H__
#define __GRAPH_NEIGHBORHOOD_H__

#include "graph.h"

typedef struct {
  int v;
  double w;
} neighbor_t;

void graph_neighbor_sort(size_t n, neighbor_t *nbrs);

/**
 * Find a neighbor of a given index in a sorted list of neighbors.
 *
 * Calls cstdlib's bsearch function for a binary search.
 *
 * @return The entry for the neighbor, or NULL if absent.
 */
neighbor_t *graph_neighbor_bsearch(size_t n, neighbor_t *nbrs, int v);

/**
 * Find a neighbor of a given index in an unsorted list of neighbors.
 *
 * Performs a linear search.
 *
 * @return The entry for the neighbor, or NULL if absent.
 */
neighbor_t *graph_neighbor_lsearch(size_t n, neighbor_t *nbrs, int v);

/**
 * Add a list of edges to a neighborhood list.
 *
 * Only sets weights where they already exist.
 *
 * @arg @c d The number of neighbors currently in the @c nbrs array.
 * @arg @c nbrs A pointer to the pointer to the beginning of the array.
 *              May be modified in reallocation.
 * @arg @c sort Whether nbrs is sorted, and to maintain that ordering.
 * @arg @c n The number of edges to add,
 * @arg @c edge A list of edges to add.
 * @arg @c fwd If true, take neighbors from the @c v element of the edges,
 *             otherwise take them from the @c u element.
 * @arg @c exclude The index of a neighbor not to add.  Ignored for -1.
 */
void graph_neighbor_list_set(
  size_t *d, neighbor_t **nbrs, bool sort,
  size_t n, const edge_t *edges, bool fwd,
  int exclude);

#endif // #ifndef __GRAPH_NEIGHBORHOOD_H__
