function [A start stop] = catcells(C, dim)
    % [A start stop] = catcells(C, dim)
    %
    %   Concatenate the elements of a cell matrix into a single matrix,
    %   and return indices into the concatenated matrix for each element
    %   of C.
    %
    %   For example, for 1-dimensional elements of C
    %
    %     A(start(i):stop(i)) == C{i}
    %

    if nargin < 2
        dim = 1;
    end

    n = numel(C);
    
    start = zeros(n, 1);
    stop = zeros(n, 1);

    A = [];
    
    start(1) = 1;
    for i=1:n
        stop(i) = start(i) + size(C{i}, dim) - 1;
        
        if i ~= n
            start(i+1) = stop(i) + 1;
        end
        
        A = cat(dim, A, C{i});
    end
end