function cluster_figure
    total_imgs = 32;

    emu_ind = randsample(53, total_imgs/2);
    pizza_ind = randsample(53, total_imgs/2);
    allind = [emu_ind; pizza_ind];
    
    % Get V1-like features
    paths = cell(total_imgs, 1);
    for i=1:(total_imgs/2)
        paths{i} = sprintf('/home/mcollins/stiefel/data/caltech101/emu/image_%04d.jpg.feat/image_%04d.jpg.v1like.mat', emu_ind(i), emu_ind(i));
    end
    
    for i=1:(total_imgs/2)
        paths{total_imgs/2 + i} = sprintf('/home/mcollins/stiefel/data/caltech101/pizza/image_%04d.jpg.feat/image_%04d.jpg.v1like.mat', pizza_ind(i), pizza_ind(i));
    end
    
    feat = zeros(total_imgs, 86400);
    for i=1:total_imgs
        s = load(paths{i});
        feat(i, :) = s.data;
    end
    
    % Perform kernel PCA
    K = feat * feat';
    X = kernel_pca(K, 2);
    
    % Normalize to [0,1]x[0,1]
    xmin = min(X(1, :));
    xmax = max(X(1, :));
    
    ymin = min(X(2, :));
    ymax = max(X(2, :));
    
    X(1, :) = (X(1, :) - xmin) / (xmax - xmin);
    X(2, :) = (X(2, :) - ymin) / (ymax - ymin);
    
    X = (X + 0.1) / 1.1;
    
    % Plot clustering for sanity check
    y = [repmat(1, 1, total_imgs/2), repmat(2, 1, total_imgs/2)];
    cluster_plot(X, y);
    
    % Write PGF code
    Xemu = X(:, 1:(total_imgs/2));
    Xpizza = X(:, (total_imgs/2 + 1):end);
    
    for i=1:(total_imgs/2)
        fprintf(1, '\\pgfdeclareimage[height=1cm]{emu%d}{fig/cluster/emu/image_%04d.jpg}\n', i, emu_ind(i));
        fprintf(1, '\\pgfputat{\\pgfxy(%f,%f)}{\\pgfbox[center,center]{\\pgfuseimage{emu%d}}}\n\n', Xemu(1, i), Xemu(2, i), i);
    end
    
    for i=1:(total_imgs/2)
        fprintf(1, '\\pgfdeclareimage[height=1cm]{pizza%d}{fig/cluster/pizza/image_%04d.jpg}\n', i, pizza_ind(i));
        fprintf(1, '\\pgfputat{\\pgfxy(%f,%f)}{\\pgfbox[center,center]{\\pgfuseimage{pizza%d}}}\n\n', Xpizza(1, i), Xpizza(2, i), i);
    end
end