function L = laplacian(W, type)
    % L = laplacian(W)
    % L = laplacian(W, type)
    %
    %   Generate a graph laplacian, including normalized variants.
    %

    if ~exist('type', 'var')
        type = 'ordinary';
    end
    
    n = size(W, 1);
    degree = sum(W);
    isolated = (degree == 0);

    if strcmp(type, 'ordinary')
        D = sparse(1:n, 1:n, degree, n, n);
        L = D - W;
    elseif strcmp(type, 'sym')
        % In this case, and rw, we need to take care that the laplacian
        % for isolated vertices is zero
        nunisolated = n - sum(isolated);
        unisolated = 1:n;
        unisolated = unisolated(~isolated);
        
        D = 1 ./ sqrt(degree(~isolated));
        D = sparse(unisolated, unisolated, D, n, n);
        
        L = sparse(unisolated, unisolated, ones(nunisolated,1), n, n);
        L = L - D * W * D;
    elseif strcmp(type, 'rw')
        nunisolated = n - sum(isolated);
        unisolated = 1:n;
        unisolated = unisolated(~isolated);
        
        D = 1 ./ degree(~isolated);
        D = sparse(unisolated, unisolated, D, n, n);
        
        L = sparse(unisolated, unisolated, ones(nunisolated,1), n, n);
        L = L - D * W;
    elseif strcmp(type, 'ng')
        D = zeros(n, 1);
        D(~isolated) = 1 ./ degree(~isolated);
        D = sparse(1:n, 1:n, D, n, n);
        L = D * W * D;
    end
end