#include "mex_interface.h"

#include "sparse_convert.h"

mxArray *mxArrayFromSparse(const cs_di *A)
{
  int j, k, m, n, nnz;

  const int *i, *p;
  const double *x;

  mxArray *mx;
  mwIndex *ir, *jc;
  double *pr;

  if(A == NULL) {
    return NULL;
  }

  // Unpack matrix
  m = A->n;
  n = A->n;

  i = A->i;
  p = A->p;
  x = A->x;

  if(A->nz != -1) {
    return NULL;
  }
  nnz = p[n];

  // Create matlab matrix
  mx = mxCreateSparse(m, n, nnz, mxREAL);
  if(mx == NULL) {
    return NULL;
  }

  jc = mxGetJc(mx);
  for(j = 0; j <= n; j++) {
    jc[j] = (mwIndex)p[j];
  }
  
  ir = mxGetIr(mx);
  pr = mxGetPr(mx);
  for(k = 0; k < nnz; k++) {
    ir[k] = (mwIndex)i[k];
    pr[k] = x[k];
  }

  return mx;
}

cs_di *mxArrayToSparse(const mxArray *mx)
{
  const mwSize *dims;

  mwIndex *ir, *jc;
  double *pr;

  int i, j, m, n, nnz;

  cs_di *A;
  
  // Check matrix format
  if(!mxIsSparse(mx)) {
    return NULL;
  }
  if(mxGetClassID(mx) != mxDOUBLE_CLASS) {
    return NULL;
  }

  // Get dimensions
  if(mxGetNumberOfDimensions(mx) != 2) {
    return NULL;
  }
  dims = mxGetDimensions(mx);
  m = (int)dims[0];
  n = (int)dims[1];

  // Unpack mxArray
  ir = mxGetIr(mx);
  jc = mxGetJc(mx);
  pr = mxGetPr(mx);
  nnz = (int)jc[n];

  // Set up CSparse array
  A = cs_di_spalloc(m, n, nnz, 1, 0);
  if(A == NULL) {
    return NULL;
  }

  for(j = 0; j <= n; j++) {
    A->p[j] = (int)jc[j];
  }

  for(i = 0; i < nnz; i++) {
    A->i[i] = (int)ir[i];
    A->x[i] = pr[i];
  }

  return A;
}
