function [W adj] = rbf_graph(X, sigma, maxdist)
    [d n] = size(X);

    if ~exist('maxdist', 'var')
        % Estimate maxdist that will estimate the complete graph,
        % with small values going to zero
        maxdist = guess_maxdist(X, 0.5);
        maxdist = -maxdist * log(1e-6);
        
        % Find maxdist that will have a maximum of nonzeros in the graph
        if n*n > 1e6
            maxdist = min(maxdist, guess_maxdist(X, 1e6 / (n*n)));
        end
    end
    

    [dist adj] = distance_graph(X, maxdist);
    %[adj dist] = knn_graph(32, X, X);

    if ~exist('sigma', 'var') || numel(sigma) == 0
        sigma = median(dist(adj ~= 0));
    end
    
    W = sparse(n, n);
    W(adj ~= 0) = exp(- dist(adj~=0).^2 / (2 * sigma^2));
end