#ifndef __STIEFEL_OPTIM_GENORTH_H__
#define __STIEFEL_OPTIM_GENORTH_H__

#include <cs.h>

#include "util/rowset.h"
#include "util/sparsemat.h"

#include "params.h"

typedef struct {
  int n, p, k;

  double (*obj)(void *user_data, const rowset *Vk);
  void (*grad)(void *user_data, const rowset *Vk, double *Gk);
  double (*fullobj)(void *user_data, int n, int p, const double *V);
  void *user_data;

  sparsemat *B;

  optim_params params;

} genorth_prob;

typedef struct genorth_ctx genorth_ctx;

genorth_ctx *genorth_ctx_init(const genorth_prob *prob);
void genorth_ctx_deinit(genorth_ctx *ctx);

/**
 * @arg @c Vout Output argument of modified V, in row-major order
 */
double genorth_step(genorth_ctx *ctx, const rowset *V, const rowset *Vnbr, const double *Gk, double *Vout);

/**
 * Basic single-threaded solver for generalized orthogonality problems.
 *
 * @arg @c V On entry, initial iterate.  On output, solution.
 *           Will be in row-major order.
 */
void genorth_solve(const genorth_prob *prob, double *V);

#endif /* #ifndef __STIEFEL_OPTIM_GENORTH_H__ */
