#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "infeas.h"

#ifndef INFINITY
#define INFINITY (1.0/0.0)
#endif

double genorth_infeas(sparsemat *B, int p, const double *V)
{
  int i, j, u, v, ind, n;
  double err, errij, Aij;
  double *VTBV;
  sparsecol col;

  n = sparsemat_nrows(B);
  assert(sparsemat_nrows(B) == sparsemat_ncols(B));

  // Calculate V' * B * V
  VTBV = (double *)malloc(sizeof(double) * p * p);
  for(i = 0; i < p*p; i++) {
    VTBV[i] = 0;
  }

  for(j = 0; j < n; j++) {
    col = sparsemat_col(B, j);

    for(ind = 0; ind < col.nnz; ind++) {
      i = col.i[ind];
      Aij = col.v[ind];

      for(u = 0; u < p; u++) {
        for(v = 0; v < p; v++) {
          VTBV[p*u + v] += Aij * V[i*p + u] * V[j*p + v];
        }
      }
    }
  }

  // Find the maximum difference between V' * B * V and the identity
  err = 0;
  for(i = 0; i < p; i++) {
    for(j = 0; j < p; j++) {
      if(i == j) {
        errij = fabs(VTBV[p*i + j] - 1);
      }
      else {
        errij = fabs(VTBV[p*i + j]);
      }

      if(errij > err) {
        err = errij;
      }
    }
  }

  free(VTBV);

  return err;
}
