#include <stdlib.h>
#include <stdio.h>
#include <sysexits.h>
#include <assert.h>

#include "util/matio.h"
#include "util/denseio.h"
#include "util/sparsemat.h"
#include "util/ident.h"

#include "infeas.h"

static const char * const USAGE = "infeas <V> [<B>]";

int main(int argc, char **argv)
{
  const char *Vpath, *Bpath;
  double *V, infeas;
  int s, n, p, Vdims[2];
  sparsemat *B;

  // Interpret arguments
  if(!(argc == 2 || argc == 3)) {
    fprintf(stderr, "Usage: %s\n", USAGE);
    return EX_USAGE;
  }

  Vpath = argv[1];
  Bpath = argc >= 3 ? argv[2] : NULL;


  // Read matrices
  V = dense_file_readall_rowmajor_path(Vpath, 1, Vdims);
  if(V == NULL) {
    fprintf(stderr, "Failed to read matrix file \"%s\"\n", Vpath);
    return EX_NOINPUT;
  }
  n = Vdims[0];
  p = Vdims[1];

  if(Bpath == NULL) {
    B = sparsemat_basic(create_ident_matrix(n), 1);
  }
  else {
    B = sparsemat_basic(sparsemat_file_read_all_path(Bpath), 1);
    assert(sparsemat_nrows(B) == n && sparsemat_ncols(B) == n);
  }

  // Calculate infeasiblity
  infeas = genorth_infeas(B, p, V);

  // Output
  printf("infeas = %lg\n", infeas);
  
  // Cleanup
  free(V);
  sparsemat_destroy(B);

  return 0;
}
