#ifndef __OPTIM_PRIOR_H__
#define __OPTIM_PRIOR_H__

#include <stdlib.h>

#include "util/intarray.h"
#include "util/rowset.h"

typedef enum {
  OBJTERM_TYPE_TRACE = 0,
  OBJTERM_TYPE_SQRT = 1
} objterm_type;

typedef struct {
  double (*obj)(void *user_data, const rowset *V, const rowset *Vnbr);
  void (*grad)(void *user_data, const rowset *V, const rowset *Vnbr, double *Gk);

  /**
   * Get the dependencies between rows for this function.
   */
  intarray *(*dependent)(void *user_data, int n, const int *inds);
  
  /**
   * Setup data
   */
  size_t (*setup_buflen)(void *user_data);
  int (*setup_serialize)(void *user_data, size_t buflen, void *buf);
  int (*setup_deserialize)(void *user_data, size_t buflen, const void *buf);

  /**
   * Determine which columns to request for a given set of selected indices.
   */
  intarray *(*indupdate_request)(void *user_data, int n, const int *inds);

  /**
   * Loading elements on the client
   */
  size_t (*indupdate_buflen)(void *user_data, int n, const int *inds);
  int (*indupdate_serialize)(void *user_data, size_t buflen, void *buf, int n, const int *inds);
  int (*indupdate_deserialize)(void *user_data, size_t buflen, const void *buf, int n, const int *inds);

  void (*destroy)(void *user_data);

  objterm_type type;
  double mult;
  void *user_data;
} termdef;

double term_obj(const termdef *def, const rowset *V, const rowset *Vnbr);
void term_grad(const termdef *def, const rowset *V, const rowset *Vnbr, double *Gk);

intarray *term_dependent(const termdef *def, int n, const int *inds);

size_t term_setup_buflen(const termdef *def);
int term_setup_serialize(const termdef *def, size_t buflen, void *buf);
int term_setup_deserialize(termdef *def, size_t buflen, const void *buf);

intarray *term_indupdate_request(const termdef *def, int n, const int *inds);
size_t term_indupdate_buflen(const termdef *def, int n, const int *inds);
int term_indupdate_serialize(const termdef *def, size_t buflen, void *buf, int n, const int *inds);
int term_indupdate_deserialize(termdef *def, size_t buflen, const void *buf, int n, const int *inds);


void term_destroy(termdef *def);



///////////////////// Manipulating the sum of all the terms ////////////////

double sumterm_obj(int nterms, const termdef * const *defs, const rowset *V, const rowset *Vnbr);
void sumterm_grad(int nterms, const termdef * const *defs, const rowset *V, const rowset *Vnbr, double *Gk);

intarray *sumterm_dependent(int nterms, const termdef * const *defs, int n, const int *inds);
intarray *sumterm_indupdate_request(int nterms, const termdef * const *defs, int n, const int *inds);

size_t sumterm_indupdate_buflen(int nterms, const termdef * const *def, int n, const int *inds);
int sumterm_indupdate_serialize(int nterms, const termdef * const *def, size_t buflen, void *buf, int n, const int *inds);
int sumterm_indupdate_deserialize(int nterms, termdef * const *def, size_t buflen, const void *buf, int n, const int *inds);

#endif /* #ifndef __OPTIM_PRIOR_H__ */
