#include <assert.h>
#include <stdlib.h>

#include "optim_config.h"
#include "config.h"

const config_params config_default_params = {
  .preload_matrices = 0,
  .presend_matrices = 0
};

static const config_var optim_vars[] = {
  {"optim.rho1", CONFIG_FLOAT},
  {"optim.epsilon", CONFIG_FLOAT},
  {"optim.max_iter", CONFIG_INT},
  {"optim.print_interval", CONFIG_INT},
  {"optim.rowsel", CONFIG_INT},

  {"matrices.preload", CONFIG_BOOL},
  {"matrices.presend", CONFIG_BOOL},

  {NULL, 0}
};

static int nvars(const config_var *v)
{
  int n;

  for(n = 0; v[n].name != NULL; n++);

  return n;
}

void optim_config_read_file(config_params *params, FILE *fp)
{
  int n;
  void **out;

  n = nvars(optim_vars);

  // Marshall config variabls
  *params = config_default_params;
  params->optim = optim_default_params;

  out = malloc(sizeof(void *) * n);
  assert(sizeof(optim_vars) == (n + 1) * sizeof(config_var)); 
  
  out[0] = &params->optim.rho1;
  out[1] = &params->optim.grad_epsilon;
  out[2] = &params->optim.max_iter;
  out[3] = &params->optim.print_interval;
  out[4] = &params->optim.rowsel;
  out[5] = &params->preload_matrices;
  out[6] = &params->presend_matrices;

  config_read_file(fp, optim_vars, out);

  free(out);
}

void optim_config_read_path(config_params *params, const char *path)
{
  FILE *fp;

  fp = fopen(path, "r");
  assert(fp != NULL);

  optim_config_read_file(params, fp);

  fclose(fp);
}
