#include <assert.h>
#include <unistd.h>
#include <sys/wait.h>

#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sysexits.h>

#include <pthread.h>

#include "client.h"

#define PORTNUM (5428)
//#define MAX_RESTARTS (32)
#define MAX_RESTARTS (1)

const char * const USAGE = "parallel-multiview-client <server> <# threads>\n";

in_addr_t server_addr;

static void *client_routine(void *arg);

int main(int argc, char **argv)
{
  struct hostent *host;
  int i, nthreads;
  pthread_t *threads;
  
  client_matrices *cm;

  if(argc < 2) {
    fprintf(stderr, "%s", USAGE);
    return EX_USAGE;
  }

  // Interpret arguments
  host = gethostbyname(argv[1]);
  if(host == NULL || host->h_addr_list[0] == NULL) {
    fprintf(stderr, "Unknown host \"%s\"\n", argv[1]);
    return EX_NOHOST;
  }
  server_addr = ((struct in_addr **)host->h_addr_list)[0]->s_addr;

  if(argc >= 3) {
    nthreads = atoi(argv[2]);
  }
  else {
    nthreads = 1;
  }

  // Set up shared data
  cm = client_matrices_alloc();

  // Start up threads
  threads = (pthread_t *)malloc(sizeof(pthread_t) * nthreads);
  for(i = 0; i < nthreads; i++) {
    pthread_create(&threads[i], NULL, client_routine, cm);
  }

  // Wait for them to complete
  for(i = 0; i < nthreads; i++) {
    pthread_join(threads[i], NULL);
  }

  // Cleanup
  client_matrices_destroy(cm);

  return 0;
}

static void *client_routine(void *arg)
{
  struct sockaddr_in serv;
  int sock, s;
  int nrestarts;

  client_matrices *cm;

  cm = (client_matrices *)arg;

  // Connect to server
  for(nrestarts = 0; nrestarts < MAX_RESTARTS; nrestarts++) {
    sock = socket(AF_INET, SOCK_STREAM, 0);
    assert(sock != -1);

    serv.sin_family = AF_INET;
    serv.sin_addr.s_addr = server_addr;
    serv.sin_port = htons(PORTNUM);

    connect(sock, (struct sockaddr *)&serv, sizeof(struct sockaddr));

    // Start up client
    s = client_run_socket(sock, cm);
    
    close(sock);

    if(s == 0) {
      break;
    }

    fprintf(stderr, "Restarting client\n");
  }

  return NULL;
}
