#ifndef __STIEFEL_OPTIM_SERIALIZE_H__
#define __STIEFEL_OPTIM_SERIALIZE_H__

#include <sys/types.h>

typedef enum {
  SERIALIZE_END = -1,

  SERIALIZE_INT = 0,
  SERIALIZE_DOUBLE,

  SERIALIZE_INT_VECTOR,
  SERIALIZE_DOUBLE_VECTOR,

  SERIALIZE_OTHER
} serial_type;

/**
 * Each tuple of variable arguments should be of the form:
 *
 *   <type>, <value>,
 *
 * unless a vector, which will be of the form
 *
 *   <type>, <length (int)>, <pointer>,
 *
 * and SERIALIZE_OTHER arguments will be
 *
 *   SERIALIZE_OTHER, <size (int)>, <pointer>,
 *
 * and should conclude with SERIALIZE_END.
 *
 * @return 0 on success, -1 on failure
 */
int serialize(size_t buflen, void *buf, ...);

/**
 * Each tuple should be of the form
 *
 *   <type>, <pointer>,
 *
 * for scalars, and the same as @c serialize otherwise.
 *
 * @see serialize
 * @return 0 on success, -1 on failure
 */
int deserialize(size_t buflen, const void *buf, ...);


/**
 * Get the length of a serialization.  Each tuple will be of the form
 *
 *   <type>,
 *
 * or 
 *
 *   <type>, <length (int)>,
 *
 * for vectors and SERIALIZE_OTHER.
 *
 * @see serialize
 * @return The necessary length of buffer to serialize the sequence.
 *         0 on error.
 */
size_t serial_length(serial_type type, ...);

#endif /* #ifndef __STIEFEL_OPTIM_SERIALIZE_H__ */
