#include <stdio.h>
#include <sysexits.h>
#include <string.h>
#include <assert.h>

#include "util/cachemat.h"
#include "util/denseio.h"
#include "sqrtterm.h"

const char * const USAGE = "sqrtobj <mustlink> <V> <bin|txt>";

int main(int argc, char **argv)
{
  const char *mustlink_path, *Vpath;
  int bin;

  mustlink *ml;
  dense_file *Vfp;
  double *V;
  int read;

  int Vdims[2], *inds, i, n;
  double sq;

  rowset *rows, *nullrows;

  // Interpret arguments
  if(argc != 4) {
    fprintf(stderr, "Usage: %s\n", USAGE);
    return EX_USAGE;
  }

  mustlink_path = argv[1];
  Vpath = argv[2];

  if(strcmp(argv[3], "txt") == 0) {
    bin = 0;
  }
  else if(strcmp(argv[3], "bin") == 0) {
    bin = 1;
  }
  else {
    fprintf(stderr, "Argument 3 must be either \"txt\" or \"bin\"\nUsage: %s\n", USAGE);
    return EX_USAGE;
  }

  // Load matrices
  ml = mustlink_read_from_path(mustlink_path);
  if(ml == NULL) {
    fprintf(stderr, "Could not load mustlink file \"%s\"\n", mustlink_path);
    return 1;
  }

  V = dense_file_readall_rowmajor_path(Vpath, bin, Vdims);
  if(V == NULL) {
    fprintf(stderr, "Could not read dense matrix file \"%s\"\n", Vpath);
    return 1;
  }

  // Check dimensions
  n = mustlink_nmembers(ml);
  if(Vdims[0] != n) {
    fprintf(stderr, "Dimension mismatch: mustlink has %d elements, V is %d x %d\n", n, Vdims[0], Vdims[1]);
    return 1;
  }

  // Calculate trace
  inds = (int *)malloc(sizeof(int) * n);
  for(i = 0; i < n; i++) {
    inds[i] = i;
  }
  rows = rowset_create(Vdims[0], Vdims[1], Vdims[0], inds, V);
  nullrows = rowset_create(Vdims[0], Vdims[1], 0, NULL, V);

  sq = sqrt_obj(ml, rows, nullrows);
  printf("sqrtobj = %lg\n", sq);

  return 0;
}
