#include <stdio.h>
#include <sysexits.h>
#include <string.h>
#include <assert.h>

#include "util/cachemat.h"
#include "util/denseio.h"
#include "trace.h"

const char * const USAGE = "traceobj <L> <V> <bin|txt>";

int main(int argc, char **argv)
{
  const char *Lpath, *Vpath;
  int bin;

  sparsemat *L;
  dense_file *Vfp;
  double *V;
  int read;

  int Ldims[2], Vdims[2];

  double tr;

  // Interpret arguments
  if(argc != 4) {
    fprintf(stderr, "Usage: %s\n", USAGE);
    return EX_USAGE;
  }

  Lpath = argv[1];
  Vpath = argv[2];

  if(strcmp(argv[3], "txt") == 0) {
    bin = 0;
  }
  else if(strcmp(argv[3], "bin") == 0) {
    bin = 1;
  }
  else {
    fprintf(stderr, "Argument 3 must be either \"txt\" or \"bin\"\nUsage: %s\n", USAGE);
    return EX_USAGE;
  }

  // Load matrices
  L = cachemat_open(Lpath);
  if(L == NULL) {
    fprintf(stderr, "Could not load sparse matrix file \"%s\"\n", Lpath);
    return 1;
  }
  sparsemat_dims(L, Ldims);

  Vfp = dense_file_open(Vpath, bin, 0);
  if(Vfp == NULL) {
    fprintf(stderr, "Could not open dense matrix file \"%s\"\n", Vpath);
    return 1;
  }
  dense_file_dims(Vfp, Vdims);

  V = (double *)malloc(sizeof(double) * Vdims[0] * Vdims[1]);
  assert(V != NULL);

  read = dense_file_readall_rowmajor(Vfp, V);
  if(read != Vdims[0] * Vdims[1]) {
    fprintf(stderr, "Error while reading dense matrix \"%s\"\n", Vpath);
    return 1;
  }

  dense_file_close(Vfp);

  // Check dimensions
  if(Ldims[0] != Ldims[1]) {
    fprintf(stderr, "L is not square: %d x %d\n", Ldims[0], Ldims[1]);
    return 1;
  }

  if(Vdims[0] != Ldims[0]) {
    fprintf(stderr, "Dimension mismatch: L is %d x %d, V is %d x %d\n", Ldims[0], Ldims[1], Vdims[0], Vdims[1]);
    return 1;
  }

  // Calculate trace
  tr = sparse_trace_full_obj(L, Vdims[1], V);
  printf("trace(V' * L * V) = %lg\n", tr);

  return 0;
}
