#include <assert.h>
#include <stdlib.h>
#include <stdint.h>
#include <memory.h>

#include "traceterm.h"
#include "trace.h"

#include "util/sort.h"
#include "util/setdiff.h"

#include "colmsg.h"
#include "serialize.h"

#include "msg.h"

#include "util/debug.h"

#ifndef MAX
#define MAX(x, y) ((x) > (y) ? (x) : (y))
#endif

static double obj(void *user_data, const rowset *V, const rowset *Vnbr)
{
  return trace_partial_obj((sparsemat *)user_data, V, Vnbr);
}

static void grad(void *user_data, const rowset *V, const rowset *Vnbr, double *Gk)
{
  return trace_partial_grad((sparsemat *)user_data, V, Vnbr, Gk);
}

static intarray *dependent(void *user_data, int n, const int *inds)
{
  int i, j, nnbr, alloc, *nbr, *onlydeps;
  sparsecol col;
  sparsemat *A;
  size_t nunique;
  
  assert(issorted(n, inds));

  A = (sparsemat *)user_data;

  nnbr = 0;
  alloc = n;
  nbr = (int *)malloc(sizeof(int) * alloc);

  for(i = 0; i < n; i++) {
    col = sparsemat_col(A, inds[i]);
    
    // Allocate
    if(nnbr + col.nnz >= alloc) {
      alloc = MAX(2*alloc, nnbr + col.nnz);
      nbr = (int *)realloc(nbr, sizeof(int) * alloc);
    }

    // Copy
    for(j = 0; j < col.nnz; j++) {
      nbr[nnbr++] = col.i[j];
    }
  }


  // Retrieve unique neighbors
  nunique = (size_t)nnbr;
  unique_ints(&nunique, nbr);
  nnbr = (int)nunique;

  // Filter out actual selected nodes
  onlydeps = (int *)malloc(sizeof(int) * nnbr);
  intsetdiff(nnbr, nbr, n, inds, &nnbr, onlydeps);

  // Cleanup
  free(nbr);

  return intarray_create_from(nnbr, onlydeps);
}


static size_t indupdate_buflen(void *user_data, int n, const int *inds)
{
  return colmsg_buflen((sparsemat *)user_data, n, inds);
}

static int indupdate_serialize(void *user_data, size_t buflen, void *buf, int n, const int *inds)
{
  return colmsg_serialize((sparsemat *)user_data, n, inds, buflen, buf);
}

void traceterm_create(termdef *def, double mult, sparsemat *A)
{
  def->obj = obj;
  def->grad = grad;

  def->dependent = dependent;

  def->setup_buflen = NULL;
  def->setup_serialize = NULL;
  def->setup_deserialize = NULL;

  def->indupdate_request = NULL;
  def->indupdate_buflen = indupdate_buflen;
  def->indupdate_serialize = indupdate_serialize;
  def->indupdate_deserialize = NULL;

  def->destroy = NULL;

  def->type = OBJTERM_TYPE_TRACE;
  def->mult = mult;
  def->user_data = A;
}
