from __future__ import print_function
import os, os.path, subprocess
import re
import distutils.spawn

from SCons.Script import *

mexre = re.compile('.*mex\s*=\s*(.*)')

def mexext(env):
    """Get extension for mex files."""

    matlab_bin = env.subst('$MATLAB')

    # Run Matlab
    cmd = [matlab_bin, '-nodisplay', '-nojvm']
    devnull = file(os.devnull)
    proc = subprocess.Popen(cmd, stdout = subprocess.PIPE, stdin = subprocess.PIPE, stderr = devnull)

    # Issue mexext command
    proc.stdin.write("fprintf(1, 'mex = %s\\n', mexext)\nexit\n")

    # Parse output
    for line in proc.stdout:
        m = mexre.match(line)
        if m:
            return m.groups()[0]


def mexGenerator(source, target, env, for_signature):
    return '$MEX $MEXFLAGS $_CPPDEFFLAGS $_CPPINCFLAGS -o $TARGET $SOURCES $_LIBDIRFLAGS $_LIBFLAGS'

def generate(env):
    try:
        matlabospath = distutils.spawn.find_executable('matlab')
        if matlabospath == None:
            matlabospath = '/usr/local/MATLAB/R2013b'
        else:
            matlabospath = os.path.dirname(os.path.dirname(matlabospath))

        env.SetDefault(MATLABDIR = matlabospath)
        env.SetDefault(MEX = '$MATLABDIR/bin/mex')
        env.SetDefault(MATLAB = '$MATLABDIR/bin/matlab')

        if not env.Detect(env['MATLAB']):
            env['WITH_MEX'] = False
            return

        env.SetDefault(MEXSUFFIX = '.' + mexext(env))
        env.SetDefault(MEXINCPATH = '$MATLABDIR/extern/include')

        if env['MEXSUFFIX'] in ['.mexa64', '.mexmaci64', '.mexw64']:
            env.PrependUnique(MEXFLAGS = ['-largeArrayDims'])

        env.AppendUnique(CPPPATH = ['$MEXINCPATH'])

        # Create builder
        bld = env.Builder(generator = mexGenerator,
                          suffix = env['MEXSUFFIX'],
                          source_scanner = CScanner)
        env.Append(BUILDERS = {'Mex': bld})

        env['WITH_MEX'] = True
    except:
        env['WITH_MEX'] = False


def exists(env):
    return True
