#include <stdlib.h>
#include <pthread.h>

#include "cachemat.h"
#include "sparsemat_internal.h"

#include <assert.h>
#include <pthread.h>
#include <stdio.h>

#include "lazymat.h"
#include "matio.h"

static const sparsemat_vtable vtable;

typedef struct
{
  sparsemat base;

  lazymat *lz;
  sparsemat_file *sm;

  pthread_rwlock_t lock;
} cachemat;

static inline cachemat *cast(sparsemat *mat)
{
  assert(mat->v == &vtable);
  return (cachemat *)mat;
}

static inline const cachemat *cast_const(const sparsemat *mat)
{
  assert(mat->v == &vtable);
  return (const cachemat *)mat;
}

sparsemat *cachemat_open(const char *path)
{
  cachemat *dm;
  int dims[2];

  dm = (cachemat *)malloc(sizeof(cachemat));
  if(dm == NULL) {
    return NULL;
  }
  dm->base.v = &vtable;

  dm->sm = sparsemat_open_binary_file(path);
  if(dm->sm == NULL) {
    free(dm);
    return NULL;
  }
  sparsemat_file_dims(dm->sm, dm->base.dims);

  dm->lz = lazymat_create(dm->base.dims);
  if(dm->lz == NULL) {
    sparsemat_close_binary_file(dm->sm);
    free(dm);
    return NULL;
  }

  pthread_rwlock_init(&dm->lock, NULL);

  return (sparsemat *)dm;
}

static void destroy(sparsemat *base)
{
  cachemat *dm;

  dm = cast(base);

  pthread_rwlock_destroy(&dm->lock);
  sparsemat_destroy((sparsemat *)dm->lz);
  sparsemat_close_binary_file(dm->sm);
  free(dm);
}

static sparsecol getcol(sparsemat *base, int j)
{
  cachemat *dm;
  sparsecol col;

  int nnz, *i;
  double *v;

  dm = cast(base);

  pthread_rwlock_rdlock(&dm->lock);
  col = sparsemat_col((sparsemat *)dm->lz, j);
  pthread_rwlock_unlock(&dm->lock);

  if(col.nnz == -1) {
    nnz = sparsemat_file_colnnz(dm->sm, j);
    i = (int *)malloc(sizeof(int) * nnz);
    v = (double *)malloc(sizeof(double) * nnz);
    if(i == NULL || v == NULL) {
      return col;
    }

    pthread_rwlock_wrlock(&dm->lock);
    sparsemat_file_get_col(dm->sm, j, i, v);
    if(sparsemat_col((sparsemat *)dm->lz, j).nnz == -1) {
      lazymat_col_add(dm->lz, j, nnz, i, v);
    }

    col = sparsemat_col((sparsemat *)dm->lz, j);
    pthread_rwlock_unlock(&dm->lock);
  }

  return col;
}

static const sparsemat_vtable vtable = {
  .destroy = destroy,
  .getcol = getcol
};
