#ifndef __OPTIM_DEBUG_H__
#define __OPTIM_DEBUG_H__

#include <sys/types.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>

/**
 * Pretty-print a dense matrix to the stream.
 */
void print_matrix(
  FILE *stream,
	const char *float_format, const char *indent,
	int rows, int cols,
	const double *A);

/**
 * Pretty-print a dense row-major matrix to the stream.
 */
void print_matrix_rowmajor(
  FILE *stream,
	const char *float_format, const char *indent,
	int rows, int cols,
	const double *A);

/**
 * Pretty-print a matrix 
 */
void print_matrix_numbered(
  FILE *stream,
	const char *float_format, const char *indent,
	int rows, int cols,
	const double *A);

struct cs_di_sparse;
void print_sparse(FILE *stream, const char *indent, struct cs_di_sparse *A);

void print_ints(FILE *stream, int n, const int *l);

int anynan(int n, const double *ds);
int issorted(int n, const int *l);

void strmemsize(size_t sz, size_t buflen, char *buf);
void printmemsize(size_t sz, FILE *stream);

size_t getmemusage();

void strtime(double sec, size_t buflen, char *buf);
void printtime(double sec, FILE *stream);
void printcurrenttime(FILE *stream);

/**
 * Returns the time elapsed, in seconds, since @c start
 */
double timeelapsed(struct timeval *start);

#endif /* #ifndef __OPTIM_DEBUG_H__ */
