#ifndef __DENSE_IO_H__
#define __DENSE_IO_H__

/**
 * File storing a dense matrix.
 */
typedef struct dense_file dense_file;

dense_file *dense_file_open(const char *path, int bin, int preload);
void dense_file_close(dense_file *df);

void dense_file_dims(dense_file *df, int dims[2]);
int dense_file_eof(dense_file *df);

int dense_file_tell(dense_file *df, int pos[2]);
int dense_file_seek(dense_file *df, int pos[2]);

double dense_file_next(dense_file *df);
int dense_file_row(dense_file *df, double *row);

/**
 * Read all contents of the file into array @c A.
 *
 * @return Number of elements read, or -1 on error.
 */
int dense_file_readall(dense_file *df, double *A);

double *dense_file_readall_path(const char *path, int bin, int dims[2]);

/**
 * Read all contents of the file into array @c A in row major order.
 *
 * @return Number of elements read, or -1 on error.
 */
int dense_file_readall_rowmajor(dense_file *df, double *A);

/**
 * @arg @c dims Output argument, dimensions of the matrix
 * @note Caller must @c free result
 * @return Matrix read from file in row-major order, or NULL on error
 */
double *dense_file_readall_rowmajor_path(const char *path, int bin, int dims[2]);

/**
 * Writing a 2D dense matrix to a file.
 *
 * @return 0 on success, -1 on failure
 */
int dense_file_write(const char *path, int bin, int m, int n, const double *A);

int dense_file_write_rowmajor(const char *path, int bin, int m, int n, const double *A);

#endif /* #ifndef __DENSE_IO_H__ */
