#ifndef __CLUSTER_GONZALEZ85_H__
#define __CLUSTER_GONZALEZ85_H__

#include <stdlib.h>

/**
 * 2-approximate min-max clustering by algorithm in Gonzalez 1985.
 *
 * @arg @c k Maximum number of clusters.
 * @arg @c mindist Distance at which we stop generating new clusters.
 *
 * @return Number of clusters created.
 */
size_t gonzalez85(size_t n, size_t ndims, const double *p,
                  size_t k, double mindist,
                  double *C, int *part);

/**
 * 2-approximate min-max clustering by algorithm in Gonzalez 1985.
 *
 * Version which allows an arbitrary metric distance.
 *
 * @arg @c exemplars The element chosen to represent this cluster.  All elements
 *                   in input data will be closer to the exemplar of their
 *                   cluster than any other exemplar.
 *                   Ignored if NULL, otherwise must have room for k elements.
 */
size_t gonzalez85_closure(
  size_t n, size_t k, double mindist, int *part, size_t *exemplars,
  double (*distance)(void *user_data, size_t i, size_t j), void *user_data);

#endif /* #ifndef __CLUSTER_GONZALEZ85_H__ */
