#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <memory.h>

#include "setdiff.h"
#include "linalg.h"

#include "indcols.h"

void indcols(int m, int n, const double *X, int *out_rank, int *ind, int *dep, double *R)
{
  int i, j, rank;
  double *Xrref;
  
  // Calculate rref form of X, extracting rank and independent columns
  Xrref = (double *)malloc(sizeof(double) * m * n);
  memcpy(Xrref, X, sizeof(double) * m * n);
  rref(m, n, 1e-8, Xrref, ind, &rank);
  *out_rank = rank;

  // Get dependent columns
  indsetdiff(n, rank, ind, dep);

  // Construct R matrix from rref
  for(j = 0; j < n - rank; j++) {
    for(i = 0; i < rank; i++) {
      R[rank*j + i] = Xrref[rank*dep[j] + ind[i]];
    }
  }

  free(Xrref);

  //fprintf(stderr, "ind/dep error = %lg\n", reconstruction_error(m, n, rank, ind, dep, X, R));
}

void select_cols(int m, int n, int k, const int *cols, const double *X, double *Xsel)
{
  int i, j;

  for(j = 0; j < k; j++) {
    assert(cols[j] >= 0 && cols[j] < n);
    assert(j <= cols[j]);

    for(i = 0; i < m; i++) {
      Xsel[m*j + i] = X[m*cols[j] + i];
    }
  }
}

void install_cols(int m, int n, int k, const int *cols, double *X, const double *Xsel)
{
  int i, j;

  for(j = 0; j < k; j++) {
    assert(cols[j] >= 0 && cols[j] < n);

    for(i = 0; i < m; i++) {
       X[m*cols[j] + i] = Xsel[m*j + i];
    }
  }
}

double reconstruction_error(int m, int n, int rank, const int *ind, const int *dep, const double *X, const double *R)
{
  double *Xind, *Xdep, *Xrec, err, diff;
  int i;

  // Select out independent and dependent columns
  Xind = (double *)malloc(sizeof(double) * m * rank);
  select_cols(m, n, rank, ind, X, Xind);

  Xdep = (double *)malloc(sizeof(double) * m * (n - rank));
  select_cols(m, n, n - rank, dep, X, Xdep);

  Xrec = (double *)malloc(sizeof(double) * m * (n - rank));

  // Multiply independent columns by transform
  dgemm_wrap('N', 'N', m, n - rank, rank, 1, Xind, m, R, rank, 0, Xrec, m);

  // Get maximum error
  err = 0;
  for(i = 0; i < m * (n - rank); i++) {
    diff = fabs(Xdep[i] - Xrec[i]);
    if(diff > err) {
      err = diff;
    }
  }

  free(Xind);
  free(Xdep);
  free(Xrec);

  return err;
}

