#ifndef __INDCOLS_H__
#define __INDCOLS_H__

/**
 * Select out the linearly independent columns of @c X
 *
 * @arg @c rank Output argument of the column rank of @c X
 * @arg @c ind Output vector, must have space for @c n entries
 * @arg @c dep Output vector, must have space for @c n - 1 entries
 * @arg @c R Output matrix, the rank x (n - rank) linear transform.
 *           Must have space for n x n entries.
 */
void indcols(int m, int n, const double *X, int *rank, int *ind, int *dep, double *R);

/**
 * Select out a subset of the columns from a column-major matrix.
 *
 * X is an m x n matrix
 * Xsel is m x k matrix
 *
 * @arg @c cols Vector of @c k column indices to select.
 */
void select_cols(int m, int n, int k, const int *cols, const double *X, double *Xsel);

/**
 * Copy @c Xsel into a subset of the columns of @c X.
 *
 * X is an m x n matrix, column-major
 * Xsel is m x k matrix, column-major
 *
 * @arg @c cols Vector of @c k column indices to select.
 */
void install_cols(int m, int n, int k, const int *cols, double *X, const double *Xsel);

double reconstruction_error(int m, int n, int rank, const int *ind, const int *dep, const double *X, const double *R);

#endif /* #ifndef __INDCOLS_H__ */
