#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include <stdio.h>

#include "sort.h"
#include "intarray.h"

struct intarray
{
  int n;
  int *v;
};

intarray *intarray_create(int n)
{
  intarray *s;

  assert(n >= 0);

  s = (intarray *)malloc(sizeof(intarray));
  if(s == NULL) {
    return NULL;
  }

  s->n = n;
  s->v = (int *)malloc(sizeof(int) * n);
  if(s->v == NULL) {
    free(s);
    return NULL;
  }

  return s;
}

intarray *intarray_create_from(int n, int *v)
{
  intarray *s;

  assert(n >= 0);

  s = (intarray *)malloc(sizeof(intarray));
  if(s == NULL) {
    return NULL;
  }

  s->n = n;
  s->v = v;

  return s;
}

intarray *intarray_clone(const intarray *I)
{
  intarray *s;

  s = intarray_create(I->n);
  if(s == NULL) {
    return NULL;
  }

  memcpy(s->v, I->v, I->n * sizeof(int));

  return s;
}

void intarray_destroy(intarray *I)
{
  if(I == NULL) {
    return;
  }

  free(I->v);
  free(I);
}

int intarray_length(const intarray *I)
{
  return I->n;
}

void intarray_resize(intarray *I, int n)
{
  I->n = n;
  I->v = realloc(I->v, sizeof(int) * n);
}

int intarray_get(const intarray *I, int i)
{
  assert(i >= 0 && i < I->n);
  return I->v[i];
}

int *intarray_contents(intarray *I)
{
  return I->v;
}

const int *intarray_contents_const(const intarray *I)
{
  return I->v;
}

void intarray_append(intarray *I, int n, const int *J)
{
  int oldn;

  oldn = I->n;
  intarray_resize(I, oldn + n);
  memcpy(I->v + oldn, J, n * sizeof(int));
}

intarray *intarray_unique(intarray *I)
{
  size_t nunique;
  int *v;

  nunique = (size_t)I->n;
  v = (int *)malloc(sizeof(int) * nunique);
  if(v == NULL) {
    return NULL;
  }
  memcpy(v, I->v, nunique * sizeof(int));

  unique_ints(&nunique, v);

  v = realloc(v, sizeof(int) * nunique);
  if(nunique > 0 && v == NULL) {
    return NULL;
  }

  return intarray_create_from((int)nunique, v);
}

void intarray_unique_update(intarray *I)
{
  size_t nunique;
  
  nunique = (size_t)I->n;
  unique_ints(&nunique, I->v);

  I->n = (int)nunique;
  I->v = realloc(I->v, sizeof(int) * nunique);
}
