#ifndef __LINALG_H__
#define __LINALG_H__

int dsyev_wrap(char jobz, char uplo, int n, double *A, int lda, double *w);

/**
 * Perform multiplication.  Calculates alpha*op(A)*op(B) + beta*C where A,B,C are matrices.
 *
 * op(A) is m x k
 * op(B) is k x n
 * C is m x n
 */
int dgemm_wrap(
  char transa, char transb,
  int m, int n, int k,
  double alpha, const double *A, int lda, const double *B, int ldb,
  double beta, double *C, int ldc);

void sqrtm_calc(int n, const double *V, const double *lambdasqrt, double *sqrtA);

/**
 * Multiply sqrtm(A) = V * diag(lambdasqrt) * V' into X
 *
 * A is m x m
 * X is m x n
 * prod is m x n
 */
void sqrtm_mult(int m, int n, const double *V, const double *lambdasqrt, const double *X, double *prod);

/**
 * Multiply sqrtm(A) = V * diag(lambdasqrt) * V' on the right of X
 *
 * A is n x n
 * X is m x n
 * prod is m x n
 */
void sqrtm_mult_right(int m, int n, const double *V, const double *lambdasqrt, const double *X, double *prod);

/**
 * Multiply inv(sqrtm(A)) = V * diag(1 ./ lambdasqrt) * V' into X
 *
 * A is m x m
 * X is m x n
 * prod is m x n
 */
void invsqrtm_mult(int m, int n, const double *V, const double *lambdasqrt, const double *X, double *solve);

/**
 * Multiply inv(sqrtm(A)) = V * diag(1 ./ lambdasqrt) * V' on the right of X
 *
 * A is n x n
 * X is m x n
 * prod is m x n
 */
void invsqrtm_mult_right(int m, int n, const double *V, const double *lambdasqrt, const double *X, double *solve);

/**
 * Solve via Cholesky decomposition
 */
void cholsolve(int n, int nrhs, const double *A, double *x);

/**
 * Solve via LU-decomposition
 *
 * @arg LU On output, the LU decomposition of A.  Ignored if NULL.
 * @arg piv On Output, the pivoting used in LU.  Ignored if NULL.
 */
int lusolve(int n, int nrhs, const double *A, double *x, double *LU, int *piv);

/**
 * Perform rref decomposition.
 */
void rref(int m, int n, double tol, double *A, int *bound, int *rank);

#endif /* #ifndef __LINALG_H__ */
