#include <assert.h>

#include "sort.h"
#include "debug.h"

#include "neighbors.h"


int *sparsemat_neighbors(int nmat, sparsemat * const *A, int k, const int *K, int *out_nnbr)
{
  int i, j, l, ind, mati, nnbr, alloc, *nbr;
  sparsecol col;
  size_t nunique;

  nnbr = 0;
  alloc = 4 * nmat * k;
  nbr = (int *)malloc(sizeof(int) * alloc);

  for(i = 0; i < k; i++) {
    for(mati = 0; mati < nmat; mati++) {
      col = sparsemat_col(A[mati], K[i]);

      // Allocate
      if(nnbr + col.nnz >= alloc) {
        alloc += col.nnz;
        nbr = (int *)realloc(nbr, sizeof(int) * alloc);
      }

      // Copy
      for(j = 0; j < col.nnz; j++) {
        nbr[nnbr++] = col.i[j];
      }
    }
  }

  // Retrieve unique neighbors
  nunique = (size_t)nnbr;
  unique_ints(&nunique, nbr);
  nnbr = (int)nunique;

  // Filter out actual selected nodes
  assert(issorted(k, K));
  assert(issorted(nnbr, nbr));

  j = 0, l = 0;
  for(i = 0; i < nnbr; ) {
    if(j == k || nbr[i] < K[j]) {
      nbr[l] = nbr[i];
      i++; l++;
    }
    else if(nbr[i] == K[j]) {
      i++; j++;
    }
    else {
      j++;
    }
  }
  nnbr = l;

  *out_nnbr = nnbr;
  return nbr;
}
