#include <string.h>
#include <stdio.h>
#include <assert.h>

#include "path.h"

static int rfind_unescaped(const char *s, char c)
{
  int i, n;

  n = strlen(s);
  for(i = n-1; i >= 0 && (s[i] != c || (i > 0 && s[i-1] == '\\')); i--);

  return i;
}

const char *path_extension(const char *path)
{
	const char *end, *ext;

  // Go to end of string
	end = ext = path + strlen(path);

  // rfind '.'
	for(; *ext != '.' && ext != path; ext--);

	if(*ext != '.') {
		return end;
	}
  ext++;

  return ext;
}

const char *path_basename(const char *path)
{
	const char *end, *base;

  // Go to end of string
	end = base = path + strlen(path);

  // rfind '/'
	for(; *base != '/' && base != path; base--); // TODO: escaped /

  if(*base == '/') {
    base++;
  }

  return base;
}

char *path_dirname(const char *path)
{
  int s;

  s = rfind_unescaped(path, '/');
  if(s == -1) {
    return strdup("");
  }
  else {
    return strndup(path, (size_t)s);
  }
}

char *path_join(const char *dir, const char *base)
{
  char *joined;
  int n, m;

  if(dir == NULL || *dir == '\0') {
    return strdup(base);
  }
  if(base == NULL || *base == '\0') {
    return strdup(dir);
  }

  n = strlen(dir);
  m = strlen(base);

  joined = (char *)malloc(n + m + 2);
  strncpy(joined, dir, n);
  joined[n] = '/';
  strncpy(joined + n + 1, base, m);
  joined[n+m+1] = '\0';

  return joined;
}
