#ifndef __PATH_H__
#define __PATH_H__

/**
 * Get the extension of the path.
 *
 * Returns those characters which appear after the last '.' character.
 * Will return the end of the string when no '.' appears.
 *
 * @arg @c path A null-terminated string.
 */
const char *path_extension(const char *path);

/**
 * Get the base name of a file.
 *
 * Returns those characters which appear after the last '/' character,
 * or the path itself if no '/' characters appear.
 *
 * @arg @c path A null-terminated string.
 */
const char *path_basename(const char *path);

/**
 * Get the directory name of a file
 *
 * Returns those characters which appear before the last (unescaped) '/'
 * character.
 *
 * @note Caller must @c free result
 */
char *path_dirname(const char *path);

/**
 * Join two paths together with a '/'
 *
 * @note Caller must @c free result
 */
char *path_join(const char *dir, const char *base);

#endif // #ifndef __PATH_H__
