#include <stdio.h>
#include <sysexits.h>
#include <stdlib.h>
#include <memory.h>
#include <assert.h>
#include <math.h>

#include "sort.h"
#include "denseio.h"

const char * const USAGE = "preclusterV <preclustering> <V>";

int *read_clustering(const char *path, int *n);

int main(int argc, char **argv)
{
  int i;
  size_t j;

  const char *precluster_path, *Vpath;
  int *precluster, n;

  size_t nunique;
  int *unique, *count;
  double *entries, *V;

  // Interpret arguments
  if(argc != 3) {
    fprintf(stderr, "Usage: %s\n", USAGE);
    return EX_USAGE;
  }

  precluster_path = argv[1];
  Vpath = argv[2];

  // Read in preclustering
  printf("Reading preclustering... "); fflush(stdout);
  precluster = read_clustering(precluster_path, &n);
  if(precluster == NULL) {
    return 1;
  }
  printf("n = %d\n", n);
  
  // Find unique ints, caculate entries
  printf("Calculating V... "); fflush(stdout);
  unique = (int *)malloc(sizeof(int) * n);
  memcpy(unique, precluster, sizeof(int) * n);
  nunique = (size_t)n;
  unique_ints(&nunique, unique);

  count = (int *)malloc(sizeof(int) * nunique);
  for(j = 0; j < nunique; j++) {
    count[j] = 0;
  }
  for(i = 0; i < n; i++) {
    count[bsearch_ints(nunique, precluster[i], unique) - unique]++;
  }

  /*
  printf("nunique = %d\n", (int)nunique);
  for(i = 0; i < (int)nunique; i++) {
    printf("%d\n", count[i]);
  }
  */

  entries = (double *)malloc(sizeof(double) * n);
  for(j = 0; j < nunique; j++) {
    entries[j] = 1.0/sqrt((double)count[j]);
  }

  V = (double *)malloc(sizeof(double) * (size_t)n * nunique);
  for(j = 0; j < (size_t)n * nunique; j++) {
    V[j] = 0;
  }
  for(i = 0; i < n; i++) {
    j = bsearch_ints(nunique, precluster[i], unique) - unique;

    V[n*j + i] = entries[j];
  }

  printf("done\n");

  // Write out result
  printf("Writing \"%s\"... ", Vpath); fflush(stdout);
  dense_file_write(Vpath, 1, n, (int)nunique, V);
  printf("done\n");

  return EX_OK;
}

int *read_clustering(const char *path, int *outn)
{
  FILE *fp;
  int *idx, i, r, n;

  fp = fopen(path, "r");
  if(fp == NULL) {
    return NULL;
  }

  n = 0;
  idx = NULL;
  r = 1;
  while(!feof(fp)) {
    r = fscanf(fp, "%d", &i);
    if(r != 1) {
      continue;
    }

    idx = realloc(idx, sizeof(int) * (n+1));
    idx[n++] = i;
  }

  *outn = n;
  return idx;
}
