#include <math.h>
#include <stdio.h>

#include "progress.h"

static void print_time(FILE *stream, time_t t)
{
	if(t < 2 * 60) {
		fprintf(stream, "%ds", (int)t);
	}
	else if(t < 5 * 60) {
		fprintf(stream, "%dm%ds", (int)(t/60), (int)(t%60));
	}
	else if(t < 2 * 60 * 60) {
		fprintf(stream, "%dm", (int)(t/60));
	}
	else if(t < 24 * 60 * 60) {
		fprintf(stream, "%dh%dm", (int)(t/(60*60)), (int)((t/60)%60));
	}
	else {
		fprintf(stream, "%dd%dh", (int)(t/(24*60*60)), (int)((t/(60*60))%(24)));
	}
}

void print_progress(FILE *stream, time_t started, int progress, int total, int decimals)
{
	char percent_format[32];
	time_t now, elapsed, remaining;
	float t;
	int i;

	now = time(NULL);
	elapsed = now - started;

	// Find appropriate percentage format
	snprintf(percent_format, 32, "%%.%df%%%%", decimals);

	// Calculations...
	if(progress > 1) {
    t = ceil(elapsed / (float)progress * (float)(total - progress));
		remaining = (time_t)t;
	}
	else {
		remaining = 0;
	}

	fprintf(stream, percent_format, 100.0 * progress / total, elapsed);

	// Print elapsed time
	fputs(" (", stream);
	print_time(stream, elapsed);
	fputs(" elapsed, est. ", stream);
	print_time(stream, remaining);
	fputs(" remaining)", stream);

	for(i = 0; i < 20; i++) {
		fputc(' ', stream);
	}

	fputc('\r', stream);
	fflush(stream);
}
