#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <sysexits.h>

#include "gonzalez85.h"
#include "denseio.h"
#include "geom.h"

static const char * const USAGE = "quantize <k> <V> <labels>\n\
\n\
Quantizes the rows of a matrix V into k clusterings, outputting the labels.\n\
";

static double distance(void *user_data, size_t i, size_t j);

int main(int argc, char **argv)
{
  int i, n, k, dims[2];
  const char *Vpath, *lblpath;
  dense_file *Vfp;
  int *part;
  FILE *partfp;

  // Interpret arguments
  if(argc != 4) {
    fprintf(stderr, "%s", USAGE);
    return EX_USAGE;
  }

  k = atoi(argv[1]);
  Vpath = argv[2];
  lblpath = argv[3];

  // Open matrix
  Vfp = dense_file_open(Vpath, 1, 0);
  if(Vfp == NULL) {
    fprintf(stderr, "Unable to open \"%s\"\n", Vpath);
    return EX_NOINPUT;
  }
  
  dense_file_dims(Vfp, dims);
  n = dims[0];

  // Perform clustering
  part = (int *)malloc(sizeof(int) * n);
  gonzalez85_closure((size_t)n, (size_t)k, 0, part, NULL, distance, Vfp);

  dense_file_close(Vfp);

  // Output clustering
  partfp = fopen(lblpath, "w+");
  if(partfp == NULL) {
    fprintf(stderr, "Unable to open \"%s\"\n", lblpath);
    return EX_CANTCREAT;
  }
  for(i = 0; i < n; i++) {
    fprintf(partfp, "%d ", part[i]);
  }
  fclose(partfp);

  // Cleanup
  free(part);

  return 0;
}

static double distance(void *user_data, size_t i, size_t j)
{
  dense_file *Vfp;
  int dims[2], pos[2];
  size_t n, d;
  double *p, *q, dist;
  
  Vfp = (dense_file *)user_data;
  dense_file_dims(Vfp, dims);
  n = (size_t)dims[0];
  d = (size_t)dims[1];

  // Extract rows
  p = (double *)malloc(sizeof(double) * d * 2);
  q = p + d;
  
  pos[0] = (int)i;
  pos[1] = 0;
  dense_file_seek(Vfp, pos);
  dense_file_row(Vfp, p);

  pos[0] = (int)j;
  dense_file_seek(Vfp, pos);
  dense_file_row(Vfp, q);

  // Calculate distance
  dist = geom_distance(d, p, q);
  
  
  // Cleanup
  free(p);

  return dist;
}
