#ifndef __RANDORTH_H__
#define __RANDORTH_H__

#include <cs.h>

/**
 * Orthogonalize n x p matrix V suing the Gram-Schmidt process.
 *
 * @arg @c B n x n sparse matrix to use for dot products: <u, v> = u' * B * v
 *           Set to NULL for the identity.
 * @arg @c Input/output matrix in col-major order
 */
void gram_schmidt(const cs_di *B, int n, int p, double *V, int fwd);

/**
 * Generate a random n x p matrix V which is orthogonal, ie V' * V = I
 */
void randorth(int n, int p, double *V);

/**
 * Generate a random n x p matrix V which is orthogonal wrt B, ie V' * B * V = I
 *
 * @arg @c B An n x n p.s.d. sparse matrix.
 */
void randgenorth(const cs_di *B, int p, double *V);


#endif /* #ifndef __RANDORTH_H__ */
