#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <string.h>
#include <assert.h>
#include <stdint.h>

#include "randorth.h"
#include "denseio.h"
#include "matio.h"

const char * const USAGE = "rorth <Vin> <Vout> <fwd|rev> [<B>]";

int main(int argc, char **argv)
{
  int n, p, fwd, dims[2];
  const char *inpath, *outpath, *Bpath;
  double *V;
  cs_di *B;

  // Interpret arguments
  if(!(argc == 4 || argc == 5)) {
    fprintf(stderr, "Usage: %s\n", USAGE);
    return EX_USAGE;
  }

  inpath = argv[1];
  outpath = argv[2];
  if(strcmp(argv[3], "fwd") == 0) {
    fwd = 1;
  }
  else if(strcmp(argv[3], "rev") == 0) {
    fwd = 0;
  }
  else {
    fprintf(stderr, "Argument 3 must be either \"fwd\" or \"rev\"\nUsage: %s\n", USAGE);
    return EX_USAGE;
  }

  Bpath = NULL;
  if(argc >= 5) {
    Bpath = argv[4];
  }

  // Read matrices
  V = dense_file_readall_path(inpath, 1, dims);
  assert(V != NULL);

  n = dims[0];
  p = dims[1];

  B = NULL;
  if(Bpath != NULL) {
    B = sparsemat_file_read_all_path(Bpath);
    assert(B != NULL);
  }

  // Orthogonalize matrix
  gram_schmidt(B, n, p, V, fwd);

  // Output
  dense_file_write(outpath, 1, n, p, V);

  // Cleanup
  free(V);

  return 0;
}
