#ifndef __ROWSET_H__
#define __ROWSET_H__

typedef struct rowset rowset;

/**
 * Create a structure to hold a subset of the rows of a matrix.
 *
 * Contains k rows from a m x n matrix.
 *
 * @arg @c ind Indices of the selected rows.
 * @arg @c X The row contents in the row-major order.
 */
rowset *rowset_create(int m, int n, int k, const int *ind, const double *X);
rowset *rowset_clone(const rowset *r);
void rowset_destroy(rowset *r);

int rowset_nrows(const rowset *r);
int rowset_nrows_all(const rowset *r);
int rowset_ncols(const rowset *r);
int rowset_nelts(const rowset *r);

const int *rowset_inds(const rowset *r);

/**
 * Get the elements of the rowset.  Will be a row-major matrix.
 */
const double *rowset_data(const rowset *r);

/**
 * Get a given row, indexed by the overall matrix.
 *
 * Returns NULL if @c i is not in the rowset
 */
const double *rowset_get(const rowset *r, int i);

/**
 * Get the row submatrix in column-major order.
 */
void rowset_getall_colmajor(const rowset *r, double *X);

/**
 * Construct a rowset by selecting a set of indices from a row-major matrix.
 */
rowset *rowset_extract(int m, int n, int k, const int *ind, const double *X);

/**
 * Install a set of rows into a larger row-major matrix.
 */
void rowset_install(double *X, const rowset *r);

#endif /* #ifndef __ROWSET_H__ */
