#include <assert.h>

#include "debug.h"
#include "sort.h"

#include "sparse_submatrix.h"


cs_di *sparse_submatrix(sparsemat *A, int nrows, const int *rowind, int ncols, const int *colind)
{
  sparsecol Aj;
  cs_di *S;
  int i, j, k, l, *subi, *subnnz, nnz;

  assert(issorted(nrows, rowind));
  assert(issorted(ncols, colind));

  // Figure out the number of nonzeros in each column of the submatrix
  subnnz = (int *)malloc(sizeof(int) * ncols);
  if(subnnz == NULL) {
    return NULL;
  }

  nnz = 0;
  for(j = 0; j < ncols; j++) {
    subnnz[j] = 0;

    Aj = sparsemat_col(A, colind[j]);
    assert(Aj.nnz >= 0); // For when using lazymat

    for(k = 0; k < Aj.nnz; k++) {
      i = Aj.i[k];

      subi = bsearch_ints((size_t)nrows, i, (int *)rowind);
      if(subi != NULL) {
        subnnz[j]++;
      }
    }

    nnz += subnnz[j];
  }

  // Allocate and set up matrix
  S = cs_di_spalloc(nrows, ncols, nnz, 1, 0);
  if(S == NULL) {
    return NULL;
  }

  S->p[0] = 0;
  for(j = 0; j < ncols; j++) {
    S->p[j+1] = S->p[j] + subnnz[j];
  }
  
  // Fill in entries
  l = 0;
  for(j = 0; j < ncols; j++) {

    Aj = sparsemat_col(A, colind[j]);

    for(k = 0; k < Aj.nnz; k++) {
      i = Aj.i[k];

      subi = bsearch_ints((size_t)nrows, i, (int *)rowind);
      if(subi != NULL) {
        S->i[l] = subi - rowind;
        S->x[l] = Aj.v[k];
        assert(S->i[l] >= 0 && S->i[l] < nrows);
        l++;
      }
    }
  }
  assert(l == nnz);

  // Cleanup
  free(subnnz);

  return S;
}

