#ifndef __SPARSEMAT_H__
#define __SPARSEMAT_H__

#include <cs.h>

/**
 * A virtual object for a sparse matrix to be accessed one column at a time.
 */

typedef struct sparsemat sparsemat;

typedef struct {
  int nnz;
  const int *i;
  const double *v;
} sparsecol;

void sparsemat_destroy(sparsemat *M);

void sparsemat_dims(const sparsemat *M, int dims[2]);
int sparsemat_nrows(const sparsemat *M);
int sparsemat_ncols(const sparsemat *M);

sparsecol sparsemat_col(sparsemat *M, int j);
double sparsemat_entry(sparsemat *M, int i, int j);

/**
 * Wrapper around suitesparse CSC matrices.
 *
 * @note @c Whether the created @c sparsemat "owns" @c M.  If true, it will
 *          call @c cs_di_spfree on @c M on destruction.  If false, @c M
 *          must be valid for the lifetime of the returnend object.
 * @note Caller must free result with @c sparsemat_destroy
 */
sparsemat *sparsemat_basic(cs_di *M, int own);

#endif /* #ifndef __SPARSEMAT_H__ */
