#include <assert.h>

#include "sparsemat_internal.h"

static const sparsemat_vtable vtable;

typedef struct {
  sparsemat base;

  cs_di *A;
  int own;

} basicsparse;

static inline basicsparse *cast(sparsemat *mat)
{
  assert(mat->v == &vtable);
  return (basicsparse *)mat;
}

static inline const basicsparse *cast_const(const sparsemat *mat)
{
  assert(mat->v == &vtable);
  return (const basicsparse *)mat;
}

sparsemat *sparsemat_basic(cs_di *A, int own)
{
  basicsparse *M;

  if(A->nz != -1) {
    return NULL;
  }

  M = (basicsparse *)malloc(sizeof(basicsparse));
  if(M == NULL) {
    return NULL;
  }
  M->base.v = &vtable;
  M->base.dims[0] = A->m;
  M->base.dims[1] = A->n;

  M->A = A;
  M->own = own;

  return (sparsemat *)M;
}

static void destroy(sparsemat *base)
{
  basicsparse *B;

  B = cast(base);
  if(B->own) {
    cs_di_spfree(B->A);
  }

  free(B);
}

static sparsecol getcol(sparsemat *M, int j)
{
  sparsecol col;
  cs_di *A;

  A = cast(M)->A;
  col.nnz = A->p[j+1] - A->p[j];
  col.i = A->i + A->p[j];
  col.v = A->x + A->p[j];

  return col;
}

static const sparsemat_vtable vtable = {
  .destroy = destroy,
  .getcol = getcol
};
